package sun.security.util;

import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import java.util.TimeZone;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Pattern;
import java.security.cert.X509Certificate;
import java.util.HashSet;
import java.util.regex.Matcher;
import java.util.StringTokenizer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Locale;
import java.security.cert.CertPathValidatorException;
import java.security.Key;
import java.security.AlgorithmParameters;
import java.security.CryptoPrimitive;
import java.util.Set;
import java.util.Iterator;
import java.util.List;

public class DisabledAlgorithmConstraints extends AbstractAlgorithmConstraints
{
    private static final Debug debug;
    public static final String PROPERTY_CERTPATH_DISABLED_ALGS = "jdk.certpath.disabledAlgorithms";
    public static final String PROPERTY_SECURITY_LEGACY_ALGS = "jdk.security.legacyAlgorithms";
    public static final String PROPERTY_TLS_DISABLED_ALGS = "jdk.tls.disabledAlgorithms";
    public static final String PROPERTY_JAR_DISABLED_ALGS = "jdk.jar.disabledAlgorithms";
    private static final String PROPERTY_DISABLED_EC_CURVES = "jdk.disabled.namedCurves";
    private final List<String> disabledAlgorithms;
    private final DisabledAlgorithmConstraints.Constraints algorithmConstraints;
    
    public DisabledAlgorithmConstraints(String s) {
        this(s, new AlgorithmDecomposer());
    }
    
    public DisabledAlgorithmConstraints(String s, AlgorithmDecomposer algorithmDecomposer) {
        super(algorithmDecomposer);
        this.disabledAlgorithms = AbstractAlgorithmConstraints.getAlgorithms(s);
        int n = -1;
        int n2 = 0;
        for (String s2 : this.disabledAlgorithms) {
            if (s2.regionMatches(true, 0, "include ", 0, 8) && s2.regionMatches(true, 8, "jdk.disabled.namedCurves", 0, "jdk.disabled.namedCurves".length())) {
                n = n2;
                break;
            }
            ++n2;
        }
        if (n > -1) {
            this.disabledAlgorithms.remove(n);
            this.disabledAlgorithms.addAll(n, AbstractAlgorithmConstraints.getAlgorithms("jdk.disabled.namedCurves"));
        }
        this.algorithmConstraints = new DisabledAlgorithmConstraints.Constraints(this.disabledAlgorithms);
    }
    
    @Override
    public final boolean permits(Set<CryptoPrimitive> set, String s, AlgorithmParameters algorithmParameters) {
        return AbstractAlgorithmConstraints.checkAlgorithm(this.disabledAlgorithms, s, this.decomposer) && (algorithmParameters == null || this.algorithmConstraints.permits(s, algorithmParameters));
    }
    
    @Override
    public final boolean permits(Set<CryptoPrimitive> set, Key key) {
        return this.checkConstraints(set, "", key, null);
    }
    
    @Override
    public final boolean permits(Set<CryptoPrimitive> set, String s, Key key, AlgorithmParameters algorithmParameters) {
        if (s == null || s.length() == 0) {
            throw new IllegalArgumentException("No algorithm name specified");
        }
        return this.checkConstraints(set, s, key, algorithmParameters);
    }
    
    public final void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
        this.permits(constraintsParameters.getAlgorithm(), constraintsParameters);
    }
    
    public final void permits(String s, Key key, AlgorithmParameters algorithmParameters, String s2) throws CertPathValidatorException {
        this.permits(s, new ConstraintsParameters(s, algorithmParameters, key, (s2 == null) ? "generic" : s2));
    }
    
    public final void permits(String s, ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
        if (constraintsParameters.getNamedCurve() != null) {
            String[] namedCurve = constraintsParameters.getNamedCurve();
            int length = namedCurve.length;
            int i = 0;
            while (i < length) {
                String str = namedCurve[i];
                if (!AbstractAlgorithmConstraints.checkAlgorithm(this.disabledAlgorithms, str, this.decomposer)) {
                    throw new CertPathValidatorException("Algorithm constraints check failed on disabled algorithm: " + str, null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
                }
                ++i;
            }
        }
        this.algorithmConstraints.permits(s, constraintsParameters);
    }
    
    public boolean checkProperty(String lowerCase) {
        lowerCase = lowerCase.toLowerCase(Locale.ENGLISH);
        Iterator<String> iterator = this.disabledAlgorithms.iterator();
        while (iterator.hasNext()) {
            if (((String)iterator.next()).toLowerCase(Locale.ENGLISH).indexOf(lowerCase) >= 0) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    private boolean checkConstraints(Set<CryptoPrimitive> set, String s, Key key, AlgorithmParameters algorithmParameters) {
        if (key == null) {
            throw new IllegalArgumentException("The key cannot be null");
        }
        if (s != null && s.length() != 0 && !this.permits(set, s, algorithmParameters)) {
            return false;
        }
        if (!this.permits(set, key.getAlgorithm(), null)) {
            return false;
        }
        String[] namedCurveFromKey = ConstraintsParameters.getNamedCurveFromKey(key);
        int length = namedCurveFromKey.length;
        int i = 0;
        while (i < length) {
            if (!this.permits(set, namedCurveFromKey[i], null)) {
                return false;
            }
            ++i;
        }
        return this.algorithmConstraints.permits(key);
    }
    
    static {
        debug = Debug.getInstance("certpath");
    }
    
    private static class Constraints
    {
        private Map<String, List<DisabledAlgorithmConstraints.Constraint>> constraintsMap;
        
        public Constraints(List<String> list) {
            this.constraintsMap = new HashMap();
            for (String s : list) {
                if (s == null) {
                    continue;
                }
                if (s.isEmpty()) {
                    continue;
                }
                String trim = s.trim();
                if (DisabledAlgorithmConstraints.debug != null) {
                    DisabledAlgorithmConstraints.debug.println("Constraints: " + trim);
                }
                int index = trim.indexOf(32);
                String hashName = AlgorithmDecomposer.hashName((index > 0) ? trim.substring(0, index) : trim);
                List value = (List<DisabledAlgorithmConstraints.Constraint>)this.constraintsMap.getOrDefault(hashName.toUpperCase(Locale.ENGLISH), new ArrayList(1));
                Iterator<String> iterator2 = AlgorithmDecomposer.getAliases(hashName).iterator();
                while (iterator2.hasNext()) {
                    this.constraintsMap.putIfAbsent(((String)iterator2.next()).toUpperCase(Locale.ENGLISH), value);
                }
                if (index <= 0 || CurveDB.lookup(trim) != null) {
                    value.add(new DisabledConstraint(hashName));
                }
                else {
                    String substring = trim.substring(index + 1);
                    Object o = null;
                    int n = 0;
                    int n2 = 0;
                    String[] split = substring.split("&");
                    Object nextConstraint;
                    for (int length = split.length, i = 0; i < length; o = nextConstraint, ++i) {
                        String trim2 = split[i].trim();
                        if (trim2.startsWith("keySize")) {
                            if (DisabledAlgorithmConstraints.debug != null) {
                                DisabledAlgorithmConstraints.debug.println("Constraints set to keySize: " + trim2);
                            }
                            StringTokenizer o2 = new StringTokenizer(trim2);
                            if (!"keySize".equals(o2.nextToken())) {
                                throw new IllegalArgumentException("Error in security property. Constraint unknown: " + trim2);
                            }
                            nextConstraint = new DisabledAlgorithmConstraints.KeySizeConstraint(hashName, DisabledAlgorithmConstraints.Constraint.Operator.of(o2.nextToken()), Integer.parseInt(o2.nextToken()));
                        }
                        else if (trim2.equalsIgnoreCase("jdkCA")) {
                            if (DisabledAlgorithmConstraints.debug != null) {
                                DisabledAlgorithmConstraints.debug.println("Constraints set to jdkCA.");
                            }
                            if (n != 0) {
                                throw new IllegalArgumentException("Only one jdkCA entry allowed in property. Constraint: " + trim);
                            }
                            nextConstraint = new jdkCAConstraint(hashName);
                            n = 1;
                        }
                        else {
                            Matcher matcher;
                            if (trim2.startsWith("denyAfter") && (matcher = DisabledAlgorithmConstraints.Constraints.Holder.DENY_AFTER_PATTERN.matcher(trim2)).matches()) {
                                if (DisabledAlgorithmConstraints.debug != null) {
                                    DisabledAlgorithmConstraints.debug.println("Constraints set to denyAfter");
                                }
                                if (n2 != 0) {
                                    throw new IllegalArgumentException("Only one denyAfter entry allowed in property. Constraint: " + trim);
                                }
                                nextConstraint = new DisabledAlgorithmConstraints.DenyAfterConstraint(hashName, Integer.parseInt(matcher.group(1)), Integer.parseInt(matcher.group(2)), Integer.parseInt(matcher.group(3)));
                                n2 = 1;
                            }
                            else if (trim2.startsWith("usage")) {
                                String[] split2 = trim2.substring(5).trim().split(" ");
                                nextConstraint = new DisabledAlgorithmConstraints.UsageConstraint(hashName, split2);
                                if (DisabledAlgorithmConstraints.debug != null) {
                                    DisabledAlgorithmConstraints.debug.println("Constraints usage length is " + split2.length);
                                }
                            }
                            else {
                                throw new IllegalArgumentException("Error in security property. Constraint unknown: " + trim2);
                            }
                        }
                        if (o == null) {
                            value.add(nextConstraint);
                        }
                        else {
                            o.nextConstraint = nextConstraint;
                        }
                    }
                }
            }
        }
        
        private List<DisabledAlgorithmConstraints.Constraint> getConstraints(String s) {
            return (List)this.constraintsMap.get(s.toUpperCase(Locale.ENGLISH));
        }
        
        public boolean permits(Key key) {
            List<DisabledAlgorithmConstraints.Constraint> constraints = this.getConstraints(key.getAlgorithm());
            if (constraints == null) {
                return true;
            }
            Iterator<Object> iterator = constraints.iterator();
            while (iterator.hasNext()) {
                if (!((DisabledAlgorithmConstraints.Constraint)iterator.next()).permits(key)) {
                    if (DisabledAlgorithmConstraints.debug != null) {
                        DisabledAlgorithmConstraints.debug.println("Constraints: failed key sizeconstraint check " + KeyUtil.getKeySize(key));
                    }
                    return false;
                }
                else {
                    continue;
                }
            }
            return true;
        }
        
        public boolean permits(String s, AlgorithmParameters obj) {
            List<DisabledAlgorithmConstraints.Constraint> constraints = this.getConstraints(s);
            if (constraints == null) {
                return true;
            }
            Iterator<Object> iterator = constraints.iterator();
            while (iterator.hasNext()) {
                if (!((DisabledAlgorithmConstraints.Constraint)iterator.next()).permits(obj)) {
                    if (DisabledAlgorithmConstraints.debug != null) {
                        DisabledAlgorithmConstraints.debug.println("Constraints: failed algorithm parameters constraint check " + obj);
                    }
                    return false;
                }
                else {
                    continue;
                }
            }
            return true;
        }
        
        public void permits(String s, ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            X509Certificate certificate = constraintsParameters.getCertificate();
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("Constraints.permits(): " + constraintsParameters.toString());
            }
            HashSet<String> set = new HashSet();
            if (s != null) {
                set.addAll(AlgorithmDecomposer.decomposeOneHash(s));
                set.add(s);
            }
            if (certificate != null) {
                set.add(certificate.getPublicKey().getAlgorithm());
            }
            if (constraintsParameters.getKey() != null) {
                set.add(constraintsParameters.getKey().getAlgorithm());
            }
            Iterator<Object> iterator = set.iterator();
            while (iterator.hasNext()) {
                List<DisabledAlgorithmConstraints.Constraint> constraints = this.getConstraints((String)iterator.next());
                if (constraints == null) {
                    continue;
                }
                Iterator<Object> iterator2 = constraints.iterator();
                while (iterator2.hasNext()) {
                    ((DisabledAlgorithmConstraints.Constraint)iterator2.next()).permits(constraintsParameters);
                }
            }
        }
        
        private static class Holder
        {
            private static final Pattern DENY_AFTER_PATTERN;
            
            static {
                DENY_AFTER_PATTERN = Pattern.compile("denyAfter\\s+(\\d{4})-(\\d{2})-(\\d{2})");
            }
        }
    }
    
    private abstract static class Constraint
    {
        String algorithm;
        DisabledAlgorithmConstraints.Constraint nextConstraint;
        
        private Constraint() {
            this.nextConstraint = null;
        }
        
        public boolean permits(Key key) {
            return true;
        }
        
        public boolean permits(AlgorithmParameters algorithmParameters) {
            return true;
        }
        
        public abstract void permits(ConstraintsParameters p0) throws CertPathValidatorException;
        
        boolean next(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            if (this.nextConstraint != null) {
                this.nextConstraint.permits(constraintsParameters);
                return true;
            }
            return false;
        }
        
        boolean next(Key key) {
            return this.nextConstraint != null && this.nextConstraint.permits(key);
        }
        
        String extendedMsg(ConstraintsParameters constraintsParameters) {
            return (constraintsParameters.getCertificate() == null) ? "." : (" used with certificate: " + constraintsParameters.getCertificate().getSubjectX500Principal() + ((constraintsParameters.getVariant() != "generic") ? (".  Usage was " + constraintsParameters.getVariant()) : "."));
        }
        
        enum Operator
        {
            EQ, 
            NE, 
            LT, 
            LE, 
            GT, 
            GE;
            
            static DisabledAlgorithmConstraints.Constraint.Operator of(String str) {
                int n = -1;
                switch (str.hashCode()) {
                    case 1952: {
                        if (str.equals("==")) {
                            n = 0;
                            break;
                        }
                        break;
                    }
                    case 1084: {
                        if (str.equals("!=")) {
                            n = 1;
                            break;
                        }
                        break;
                    }
                    case 60: {
                        if (str.equals("<")) {
                            n = 2;
                            break;
                        }
                        break;
                    }
                    case 1921: {
                        if (str.equals("<=")) {
                            n = 3;
                            break;
                        }
                        break;
                    }
                    case 62: {
                        if (str.equals(">")) {
                            n = 4;
                            break;
                        }
                        break;
                    }
                    case 1983: {
                        if (str.equals(">=")) {
                            n = 5;
                            break;
                        }
                        break;
                    }
                }
                switch (n) {
                    case 0: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.EQ;
                    }
                    case 1: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.NE;
                    }
                    case 2: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.LT;
                    }
                    case 3: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.LE;
                    }
                    case 4: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.GT;
                    }
                    case 5: {
                        return DisabledAlgorithmConstraints.Constraint.Operator.GE;
                    }
                    default: {
                        throw new IllegalArgumentException("Error in security property. " + str + " is not a legal Operator");
                    }
                }
            }
        }
    }
    
    private static class jdkCAConstraint extends DisabledAlgorithmConstraints.Constraint
    {
        jdkCAConstraint(String algorithm) {
            this.algorithm = algorithm;
        }
        
        @Override
        public void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("jdkCAConstraints.permits(): " + this.algorithm);
            }
            if (constraintsParameters.isTrustedMatch()) {
                if (this.next(constraintsParameters)) {
                    return;
                }
                throw new CertPathValidatorException("Algorithm constraints check failed on certificate anchor limits. " + this.algorithm + this.extendedMsg(constraintsParameters), null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
            }
            else {
                return;
            }
        }
    }
    
    private static class DenyAfterConstraint extends DisabledAlgorithmConstraints.Constraint
    {
        private Date denyAfterDate;
        private static final SimpleDateFormat dateFormat;
        
        DenyAfterConstraint(String algorithm, int i, int n, int j) {
            this.algorithm = algorithm;
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("DenyAfterConstraint read in as:  year " + i + ", month = " + n + ", day = " + j);
            }
            Calendar build = new Calendar.Builder().setTimeZone(TimeZone.getTimeZone("GMT")).setDate(i, n - 1, j).build();
            if (i > build.getActualMaximum(1) || i < build.getActualMinimum(1)) {
                throw new IllegalArgumentException("Invalid year given in constraint: " + i);
            }
            if (n - 1 > build.getActualMaximum(2) || n - 1 < build.getActualMinimum(2)) {
                throw new IllegalArgumentException("Invalid month given in constraint: " + n);
            }
            if (j > build.getActualMaximum(5) || j < build.getActualMinimum(5)) {
                throw new IllegalArgumentException("Invalid Day of Month given in constraint: " + j);
            }
            this.denyAfterDate = build.getTime();
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("DenyAfterConstraint date set to: " + DisabledAlgorithmConstraints.DenyAfterConstraint.dateFormat.format(this.denyAfterDate));
            }
            return;
        }
        
        @Override
        public void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            Date date;
            String str;
            if (constraintsParameters.getJARTimestamp() != null) {
                date = constraintsParameters.getJARTimestamp().getTimestamp();
                str = "JAR Timestamp date: ";
            }
            else if (constraintsParameters.getPKIXParamDate() != null) {
                date = constraintsParameters.getPKIXParamDate();
                str = "PKIXParameter date: ";
            }
            else {
                date = new Date();
                str = "Current date: ";
            }
            if (!this.denyAfterDate.after(date)) {
                if (this.next(constraintsParameters)) {
                    return;
                }
                throw new CertPathValidatorException("denyAfter constraint check failed: " + this.algorithm + " used with Constraint date: " + DisabledAlgorithmConstraints.DenyAfterConstraint.dateFormat.format(this.denyAfterDate) + "; " + str + DisabledAlgorithmConstraints.DenyAfterConstraint.dateFormat.format(date) + this.extendedMsg(constraintsParameters), null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
            }
            else {
                return;
            }
        }
        
        @Override
        public boolean permits(Key key) {
            if (this.next(key)) {
                return true;
            }
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("DenyAfterConstraints.permits(): " + this.algorithm);
            }
            return this.denyAfterDate.after(new Date());
        }
        
        static {
            dateFormat = new SimpleDateFormat("EEE, MMM d HH:mm:ss z yyyy");
        }
    }
    
    private static class UsageConstraint extends DisabledAlgorithmConstraints.Constraint
    {
        String[] usages;
        
        UsageConstraint(String algorithm, String[] usages) {
            this.algorithm = algorithm;
            this.usages = usages;
        }
        
        @Override
        public void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            String[] usages = this.usages;
            int length = usages.length;
            int i = 0;
            while (i < length) {
                String str = usages[i];
                String s = null;
                if (str.compareToIgnoreCase("TLSServer") == 0) {
                    s = "tls server";
                }
                else if (str.compareToIgnoreCase("TLSClient") == 0) {
                    s = "tls client";
                }
                else if (str.compareToIgnoreCase("SignedJAR") == 0) {
                    s = "plugin code signing";
                }
                if (DisabledAlgorithmConstraints.debug != null) {
                    DisabledAlgorithmConstraints.debug.println("Checking if usage constraint \"" + s + "\" matches \"" + constraintsParameters.getVariant() + "\"");
                    ByteArrayOutputStream out = new ByteArrayOutputStream();
                    new Exception().printStackTrace(new PrintStream(out));
                    DisabledAlgorithmConstraints.debug.println(out.toString());
                }
                if (constraintsParameters.getVariant().compareTo(s) == 0) {
                    if (this.next(constraintsParameters)) {
                        return;
                    }
                    throw new CertPathValidatorException("Usage constraint " + str + " check failed: " + this.algorithm + this.extendedMsg(constraintsParameters), null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
                }
                else {
                    ++i;
                }
            }
        }
    }
    
    private static class KeySizeConstraint extends DisabledAlgorithmConstraints.Constraint
    {
        private int minSize;
        private int maxSize;
        private int prohibitedSize;
        private int size;
        
        public KeySizeConstraint(String algorithm, DisabledAlgorithmConstraints.Constraint.Operator operator, int maxSize) {
            this.prohibitedSize = -1;
            this.algorithm = algorithm;
            switch (operator) {
                case EQ: {
                    this.minSize = 0;
                    this.maxSize = Integer.MAX_VALUE;
                    this.prohibitedSize = maxSize;
                    break;
                }
                case NE: {
                    this.minSize = maxSize;
                    this.maxSize = maxSize;
                    break;
                }
                case LT: {
                    this.minSize = maxSize;
                    this.maxSize = Integer.MAX_VALUE;
                    break;
                }
                case LE: {
                    this.minSize = maxSize + 1;
                    this.maxSize = Integer.MAX_VALUE;
                    break;
                }
                case GT: {
                    this.minSize = 0;
                    this.maxSize = maxSize;
                    break;
                }
                case GE: {
                    this.minSize = 0;
                    this.maxSize = ((maxSize > 1) ? (maxSize - 1) : 0);
                    break;
                }
                default: {
                    this.minSize = Integer.MAX_VALUE;
                    this.maxSize = -1;
                    break;
                }
            }
        }
        
        @Override
        public void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            Key key = null;
            if (constraintsParameters.getKey() != null) {
                key = constraintsParameters.getKey();
            }
            else if (constraintsParameters.getCertificate() != null) {
                key = constraintsParameters.getCertificate().getPublicKey();
            }
            if (key != null && !this.permitsImpl(key)) {
                if (this.nextConstraint != null) {
                    this.nextConstraint.permits(constraintsParameters);
                    return;
                }
                throw new CertPathValidatorException("Algorithm constraints check failed on keysize limits. " + this.algorithm + " " + KeyUtil.getKeySize(key) + "bit key" + this.extendedMsg(constraintsParameters), null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
            }
            else {
                return;
            }
        }
        
        @Override
        public boolean permits(Key key) {
            if (this.nextConstraint != null && this.nextConstraint.permits(key)) {
                return true;
            }
            if (DisabledAlgorithmConstraints.debug != null) {
                DisabledAlgorithmConstraints.debug.println("KeySizeConstraints.permits(): " + this.algorithm);
            }
            return this.permitsImpl(key);
        }
        
        @Override
        public boolean permits(AlgorithmParameters algorithmParameters) {
            String algorithm = algorithmParameters.getAlgorithm();
            if (!this.algorithm.equalsIgnoreCase(algorithmParameters.getAlgorithm()) && !AlgorithmDecomposer.getAliases(this.algorithm).contains(algorithm)) {
                return true;
            }
            int keySize = KeyUtil.getKeySize(algorithmParameters);
            return keySize != 0 && (keySize <= 0 || (keySize >= this.minSize && keySize <= this.maxSize && this.prohibitedSize != keySize));
        }
        
        private boolean permitsImpl(Key key) {
            if (this.algorithm.compareToIgnoreCase(key.getAlgorithm()) != 0) {
                return true;
            }
            this.size = KeyUtil.getKeySize(key);
            return this.size != 0 && (this.size <= 0 || (this.size >= this.minSize && this.size <= this.maxSize && this.prohibitedSize != this.size));
        }
    }
    
    private static class DisabledConstraint extends DisabledAlgorithmConstraints.Constraint
    {
        DisabledConstraint(String algorithm) {
            this.algorithm = algorithm;
        }
        
        @Override
        public void permits(ConstraintsParameters constraintsParameters) throws CertPathValidatorException {
            throw new CertPathValidatorException("Algorithm constraints check failed on disabled algorithm: " + this.algorithm + this.extendedMsg(constraintsParameters), null, null, -1, CertPathValidatorException.BasicReason.ALGORITHM_CONSTRAINED);
        }
        
        @Override
        public boolean permits(Key key) {
            return false;
        }
    }
}
